package jfd.ale;

import java.lang.reflect.Field;
import java.awt.Container;
import java.awt.Component;
import java.util.Hashtable;
import java.util.Vector;

public class ALE
{
   private ALE() {}

   /**
    * Process the requested section in the supplied filename,
    * adding Components referenced in the section to the supplied
    * Container
    *
    * @param fileName file name containing the ale layout information
    * @param sectionName section to process within the supplied filename
    * @param enclosingObject object implementing the FieldToComponentMapper
    * interface that allows this method to get references to the Components
    * specified in the requested section.
    * @param theContainer Container into which the specified Components will
    * be placed.
    * @exception Exception if a fatal error is encountered while processing
    * the requested file and section.  The thrown Exception's message
    * String will contain details as to the nature of the problem.
    */

   public static void brew(
      String fileName,
      String sectionName,
      FieldToComponentMapper enclosingObject,
      Container theContainer
   )
      throws Exception
   {
      processInternal(
         Parser.getSection(fileName, sectionName), enclosingObject, theContainer
      );
   }

   /**
    * Process the requested section in the supplied String array,
    * adding Components referenced in the section to the supplied
    * Container
    *
    * @param array String[] containing the ale layout information
    * @param sectionName section to process within the supplied array
    * @param enclosingObject object implementing the FieldToComponentMapper
    * interface that allows this method to get references to the Components
    * specified in the requested section.
    * @param theContainer Container into which the specified Components will
    * be placed.
    * @exception Exception if a fatal error is encountered while processing
    * the requested String array[] and section.  The thrown Exception's message
    * String will contain details as to the nature of the problem.
    */

   public static void brew(
      String array[],
      String sectionName,
      FieldToComponentMapper enclosingObject,
      Container theContainer
   )
      throws Exception
   {
      processInternal(
         Parser.getSection(array, sectionName), enclosingObject, theContainer
      );
   }

   private static void processInternal(
      Section section,
      FieldToComponentMapper enclosingObject,
      Container theContainer
   )
      throws Exception
   {
      if(theContainer == null)
      {
         throw new Exception(
            "null Container passed to ALE.brew"
         );
      }

      Class theClass            = enclosingObject.getClass();
      Hashtable nameToComponent = new Hashtable();

      while(true)
      {
         Field fields[] = null;

         /*******************************************************************
          * It seems some AppletSecurityManagers are so draconian that
          * the won't let you even get Field objects for non public fields.
          * Nevermind the fact that a Field object for a member variable
          * is by no means an actual *reference* to that variable.  Anyway,
          * as a compromise to allow ALE to work in applets, try first
          * to getDeclaredFields and if that throws a SecurityException
          * back off to just using getField.
          *
          * Using "getField" means that any component referenced in a .ale
          * file must be public in "enclosingObject" or public in a
          * base class of "enclosingObject".  It also means that
          * we could call field.get(Object) to obtain a reference directly
          * instead of having to go through FieldToComponentMapper but
          * we'll not worry about that for the moment.
          *******************************************************************/

         try
         {
            fields = theClass.getDeclaredFields();
         }
         catch(SecurityException e)
         {
            fields = theClass.getFields();
         }

         for(int i = 0; i < fields.length; i ++)
         {
            // We are only interested in those fields that are
            // subclasses of Component

            if(Component.class.isAssignableFrom(fields[i].getType()))
            {
               try
               {
                  nameToComponent.put(
                     fields[i].getName(),
                     enclosingObject.mapToComponent(fields[i])
                  );
               }
               catch(Exception e)
               {
                  // This component is not accessible to us
               }
            }
         }

         if((theClass = theClass.getSuperclass()) == null)
            break;
      }
      
      ConstraintLayout layout = new ConstraintLayout(enclosingObject);
      theContainer.setLayout(layout);

      Constraints theConstraints[] =
         section.getConstraints(
            enclosingObject,
            (MethodTableReciever)layout
         );

      for(int i = 0; i < theConstraints.length; i ++)
      {
         String name = theConstraints[i].getName();
         Component c = (Component)nameToComponent.get(name);

         if(c == null)
         {
            System.out.println(
               "Unable to obtain a reference to Component \"" + name + "\""
            );

            System.out.println(
               "either the supplied FieldToComponentMapper object does not"
            );

            System.out.println(
               "contain this Component as a field or its value is null"
            );

            System.exit(-1);
         }

         theContainer.add(c, theConstraints[i]);
      }
   }
}
