package jfd.ale;

class GridResource extends ConstraintsContainer
{
   private int rows    = 0;
   private int columns = 0;
   private Node xMin   = null;
   private Node yMin   = null;
   private Node xMax   = null;
   private Node yMax   = null;

   public GridResource(
      int rows,
      int columns,
      Node xMin,
      Node yMin,
      Node xMax,
      Node yMax
   )
   {
      this.rows    = rows;
      this.columns = columns;
      this.xMin    = xMin;
      this.yMin    = yMin;
      this.xMax    = xMax;
      this.yMax    = yMax;
   }

   public Constraints[] getConstraints()
   {
      Constraints array[] = super.getConstraints();

      // The formula for the location (in x) of a Component in
      // column [c] is:
      //
      // xMin + (c + 1) * (xMax - xMin) / (columns + 1)
      //
      // The formula for the location (in y) of a Component in
      // row [r] is:
      //
      // yMin + (r + 1) * (yMax - yMin) / (rows + 1)

      String xBefore = new String(xMin + "+");
      String yBefore = new String(yMin  + "+");

      String xAfter = new String(
         "*(" + xMax + "-" + xMin + ")/" + (columns + 1)
      );

      String yAfter = new String(
         "*(" + yMax + "-" + yMin + ")/" + (rows + 1)
      );

      for(int i = 0; i < array.length; i ++)
      {
         int row    = i / columns;
         int column = i % columns;

         String locationString =
            xBefore + (column + 1) + xAfter + "," + 
            yBefore + (row + 1)    + yAfter;

         try
         {
            NodePair pair = Parser.parseNodePair(locationString);
            
            array[i] = new Constraints(
               array[i].getName(),
               array[i].getWidth(),
               array[i].getHeight(),
               pair.getFirst(),
               pair.getSecond(),
               array[i].getAlign()
            );
         }
         catch(Exception e)
         {
            // If we are here, it is an internal error -- the
            // String we created for "locationString" had a
            // syntax error

            System.out.println(
               "GridResource - problem parsing: [" + locationString + "]"
            );

            System.out.println(e.getMessage());
         }
      }

      return(array);
   }
}
