package jfd.ale;

import java.util.Vector;
import java.util.Hashtable;

class Section extends ConstraintsContainer
{
   private String section = null;

   public Section(String section)
   {
      this.section = section;
   }

   public String toString()
   {
      StringBuffer sb = new StringBuffer("");

      for(int j = container.size() - 1, i = 0; j >= 0; j --, i ++)
         sb.append((i > 0 ? "\n\n" : "") + container.elementAt(i));

      return(sb.toString());
   }

   public Constraints[] getConstraints(
      Object enclosingObject,
      MethodTableReciever reciever
   )
   {
      Constraints array[] = super.getConstraints();

      // Verify that no undefined Components are referenced

      Hashtable definedComponents = new Hashtable();

      for(int i = 0; i < array.length; i ++)
         definedComponents.put(array[i].getName(), array[i].getName());

      // Components are also allowed to refer to "parent" and "this",
      // add those names as well

      definedComponents.put("parent", "parent");
      definedComponents.put("this",   "this");

      for(int i = 0; i < array.length; i ++)
      {
         String name = array[i].getName();

         array[i].getWidth().accept(
            new DefinitionMethodVisitor(
               definedComponents, section, name, ConstraintLayout.WIDTH
            )
         );

         array[i].getHeight().accept(
            new DefinitionMethodVisitor(
               definedComponents, section, name, ConstraintLayout.HEIGHT
            )
         );

         array[i].getX().accept(
            new DefinitionMethodVisitor(
               definedComponents, section, name, ConstraintLayout.X
            )
         );

         array[i].getY().accept(
            new DefinitionMethodVisitor(
               definedComponents, section, name, ConstraintLayout.Y
            )
         );
      }

      // Now check out the size and location fields

      definedComponents.clear();

      // Again, all Components are free to refer to "parent"

      definedComponents.put("parent", "parent");

      for(int i = 0; i < array.length; i ++)
      {
         String name = array[i].getName();

         array[i].getWidth().accept(
            new SizeMethodVisitor(
               definedComponents,
               section,
               name,
               ConstraintLayout.WIDTH,
               new String[] {}
            )
         );

         array[i].getHeight().accept(
            new SizeMethodVisitor(
               definedComponents,
               section,
               name,
               ConstraintLayout.HEIGHT,
               new String[] { ConstraintLayout.WIDTH }
            )
         );

         array[i].getX().accept(
            new LocationMethodVisitor(
               definedComponents,
               section,
               name,
               ConstraintLayout.X,
               new String[] {}
            )
         );

         array[i].getY().accept(
            new LocationMethodVisitor(
               definedComponents,
               section,
               name,
               ConstraintLayout.Y,
               new String[]
               {
                  ConstraintLayout.X,
                  ConstraintLayout.LEFT,
                  ConstraintLayout.RIGHT
               }
            )
         );

         definedComponents.put(name, name);
      }

      // Lastly, ensure the existence and type-correctness of all
      // external methods

      Hashtable methodNameToMethod = new Hashtable();

      for(int i = 0; i < array.length; i ++)
      {
         array[i].getWidth().accept(
            new ExternalMethodVisitor(enclosingObject, methodNameToMethod)
         );

         array[i].getHeight().accept(
            new ExternalMethodVisitor(enclosingObject, methodNameToMethod)
         );

         array[i].getX().accept(
            new ExternalMethodVisitor(enclosingObject, methodNameToMethod)
         );

         array[i].getY().accept(
            new ExternalMethodVisitor(enclosingObject, methodNameToMethod)
         );
      }

      reciever.recieve(methodNameToMethod);

      return(array);
   }
}
